//Class file to store and manipulate the data depending on defined variables 
//within the storage devices file structure.

//Media class (abstract)
abstract class Media {
  public String name;
  public String path;
  public String filesize;
  public String modified;
  public String created;
  public String credits;
  String[] elements;
  int xpos, ypos;
  PFont font;
  int fontSize, colour;
  float backgroundC;
  PImage imagefile;

  Media(String[] elements) {
    this.name = elements[0];
    this.filesize = elements[1];
    this.modified = elements[2];
    this.path = elements[3];
    this.created = elements[4];
    this.credits = elements[5];
  }

  void build() {
  };
  
  //Draw the meta data read from the storage device.
  void drawMetaMultiple(int xpos, int ypos, PFont font, int fontSize, int colour, int w, int h) {
    String thisClass = new String(this.getClass().toString());
    fill(colour);
    textFont(font, fontSize); 
    text(name,xpos,ypos+250,w,h);
    text(filesize,xpos,ypos+500,w,h);
    text(modified,xpos,ypos+742,w,h);
  };
  
  //Draws meta just once!
  void drawMetaSingle(int xpos, int ypos, PFont font, int fontSize, int colour, int w, int h) {
    String thisClass = new String(this.getClass().toString());
    fill(colour);
    textFont(font, fontSize); 
    text(path,xpos+320,ypos+820,w,h);
    text(created,xpos,ypos,w,h);
    text(credits,xpos+450,ypos,w,h);
  }

  //A messy function to determine the time of day the volume is read and 
  //set the background colour reflecting the level of light.
  public float timeOfDay() {
    float s = second()*0.01; 
    float m = minute()*0.01;
    float h = hour();
    float total = (s+m+h)*(h+100);
    if (total <= 1480) {
      backgroundC = map(total, 0, 1480, 0, 255);
    } 
    else {
      backgroundC = map(total, 1400, 3000, 255, 0);
    }
    return backgroundC;
  }
} 

//The TextClass
class TextMedia extends Media {
  String[] lines;
  String[] input;
  String[] textlines;
  int w, h, r;

  TextMedia(String[] elements, String[] lines) {
    super(elements);
    build(lines);
  }

  public void build(String[] input) {
    textlines = new String[input.length];
    for (int i = 0; i<input.length; i++) {
      textlines[i] = input[i];
    }
  }
  
  //Draws any text read from txt files etc, off the storage device
  public void drawMedia(PFont font, int fontSize, int xpos, int ypos, int w, int h) {
    this.font = font;
    this.fontSize = fontSize;
    this.xpos = xpos;
    this.ypos = ypos;
    this.w = w;
    this.h = h;

    font = createFont(changeFont(), fontSize);
    fill(255);
    r = int(random(textlines.length));
    textFont(font, fontSize);
    text(textlines[r], xpos, ypos+(r*10), w, h);
  }
  
  //Function to change the font depending on the amount of uppercase or lowercase
  //letters in the names of the files in the file structure.
  public String changeFont() {
    String x = name;
    String fontname = "";
    int uppercase = 0, lowercase = 0;
    for(int i = 0; i < x.length(); i++) {
      if(Character.isUpperCase(x.charAt(i))) {
        uppercase++;
      } 
      else if (Character.isLowerCase(x.charAt(i))) {
        lowercase++;
      }
    }
    
    //NOTE: DEFINE THESE TO THE FONTS YOU HAVE ON YOUR SYSTEM
    switch (uppercase) {
    case 0:  
      fontname = "Comfortaa-Thin"; 
      break; 
    case 1:  
      fontname = "saxMono"; 
      break;
    case 2:  
      fontname = "AndaleMono"; 
      break;
    case 3:  
      fontname = "InterFaceCyr"; 
      break;
    case 4:  
      fontname = "DejaVuSerif"; 
      break;
    case 5:  
      fontname = "Georgia"; 
      break;
    case 6:  
      fontname = "GaramondPremrPro"; 
      break;
    default: 
      fontname = "saxMono";
      break;
    }
    if (uppercase > 6) {
      fontname = "BickhamScriptPro-Regular";
    } 
    else {
      return fontname;
    }

    return fontname;
  }
}

//The ImageClass
class ImageMedia extends Media {
  PImage imagefile;
  String imgpath;
  int maxpal = 255;
  int numpal = 0;
  color[] aColour = new color[maxpal];
  color fillc;
  int f, count;

  ImageMedia(String[] elements, String imgpath) {
    super(elements);
    build(imgpath);
  }

  void build(String imgpath) {
    imagefile = loadImage(imgpath);
  }

  public void drawMedia(int xpos, int ypos) {
    //tint(fillc);
    image(imagefile, xpos, ypos);
    noTint();
  }

  //Messy functions to get a single average colour from an image
  public void averageColour(PImage img) {
    f = int(random(0,9));
    takecolour(img);
    color fillc = theColour();
    fill(fillc);
    noStroke();

  }

  color theColour() {
    return aColour[int(random(numpal))];
  }

  private void takecolour(PImage fn) {

    for (int x=0;x<fn.width;x++) {
      for (int y=0;y<fn.height;y++) {
        color c = fn.get(x,y);
        boolean exists = false;
        for (int n=0;n<numpal;n++) {
          if (c==aColour[n]) {
            exists = true;
            break;
          }
        }
        if (!exists) {
          if (numpal<maxpal) {
            aColour[numpal] = (c & 0xffffff) | (postAmount()*5) << 24;
            numpal++;
          }
        }
      }
    }
  }
}

//Time for the funk. Sketches within sketches. This is quite experimental so
//good luck to you, one and all ;D
class Sketches {
  PApplet[] sketches;
  String[] names;
  int s = 0;
  int backgroundColour;
  
  
  Sketches(int backgroundColour){
    this.backgroundColour = backgroundColour;
  }

  void setup() {

    size(W, H);
    Class[] classes = getClass().getDeclaredClasses();
    int n = classes.length;
    sketches = new PApplet[n];
    names = new String[n];

    for(int i=0; i<n; i++) {
      try {
        Constructor c = classes[i].getDeclaredConstructor(getClass());
        sketches[i] = (PApplet) c.newInstance(this);
        sketches[i].init();  
        names[i] = split(c.getName(), "$")[1];
      } 
      catch (Exception e) {
      }
    }

    //frameRate(fps);

    // start next sketch
    println("starting " + names[s] + s);
    sketches[s].run();
  }


  void draw() {

    sketches[s].handleDraw();
    sketchimage = sketches[s].get();
    sketchimage.loadPixels();
    image(sketchimage, 0, 0);

    //if(export && frameCount <= fps*seconds*sketches.length) saveFrame("showreel-#####.png");
  } 

  class Sketch1 extends PApplet {
    void setup() {
      size(W, H);
      smooth();
      noStroke();
      background(backgroundColour); 
    }
    void draw() {
      ellipse(W/2, H/2, 400, 400);
    }
  }

 class Sketch2 extends PApplet {
    void setup() {
      size(W, H, P2D);
    }
    void draw() {
      background(backgroundColour); 
      ellipse(0, 0, 100, 100);
    }
  }
/*

  class Sketch3 extends PApplet {
    void setup() {
      size(W, H, P3D);
    }
    void draw() {
      background(BG); 
      lights(); 
      noStroke(); 
      translate(W/2, H/2, 0); 
      sphere(10);
    }
  }*/
}
