/*
 * Mass Responsive Device
 * by Mac Oosthuizen.
 *
 * Reads the data from an inserted USB Mass Storage Device, 
 * creates an image file representing a map of your device and sends it
 * to print at the specified printer.
 * Thanks to the staff at CIID & visiting faculty, Dennis Paul and Patrick 
 * Kochlik at The Product, Berlin.
 *
 *
 * NOTE: This works on Mac but I can't confirm it works on PC/Windows and
 * I have not been able to test it so use at your own risk.
 *
 * Enjoy!
 */

import processing.opengl.*; 
import eu.medsea.mimeutil.*;
import org.slf4j.*;
import org.apache.commons.io.filefilter.HiddenFileFilter;
import java.lang.reflect.Constructor;

//Sets the variables for the window height, weight and background colour
final int W = 595, H = 842, BG = #FFFFFF;

PImage sketchimage;
int maxImages = 10; // Max number of elements
int maxText = 60;

//Array declaration for images and text from the storage device to be stored locally
ArrayList MediaArray;
Media[] imgRecord = new Media[maxImages];
Media[] txtRecord = new Media[maxText];
String typeString;
int x;
int y;

PFont bigFont;
PFont midFont;
PFont regularFont;
PFont metaFont;

//This defines what extentions the program looks for on the USB. All supported extentions
//have been defined.
String[] pdelines;
String[] imgExt = {
  ".gif", ".jpg", ".tga", ".png", ".GIF", ".JPG", ".TGA", ".PNG", ".jpeg", ".JPEG"
};

//Change to the desired font
String font = "saxMono";

boolean success;

//Declares the printer path & name to which to send it. CHANGE THIS!
String PrinterAppName = "Brother HL-2270DW series-1";  //your printqueue name here
String pathToPrinter = "/Users/Mac/Library/Printers/";  //your printer location here
String USB;

File[] mOldVolumes;
Sketches sketch;

void setup() {
  size(W, H);
  background(BG);
  noStroke();
  bigFont = createFont(font, 350, true);
  midFont = createFont(font, 45, true); 
  regularFont = createFont(font, 16, true);
  metaFont = createFont(font, 11, true); 
  MediaArray = new ArrayList();
  File volumes = new File("/Volumes");  
  mOldVolumes = volumes.listFiles(); 

  //Un-comment the next two lines if you want to experiment with embedded sketches. See DrawPoster file
  //sketch.build(sketch.n, sketch.classes);
  //sketch.drawSketch();
  
}

//Pressing a key updates the volume list to which we check if a new volume has been inserted
void keyPressed() {
  if (key==' ') {
    println("+++ updating volume list");
    File volumes = new File("/Volumes");  
    mOldVolumes = volumes.listFiles();
  }
}

//Run the main function and if it is a success then eject the volume through the
//CMD line
void draw() { 
  mainRun();
  if(success) {
    String[] ejectCMD = { 
      "diskutil", "eject", USB
    };
    executeCMD(ejectCMD);
  }
  
}

//MAIN: Check for USB drives, run main sketch and print poster
private boolean mainRun() {
  success = false;
  File volumes = new File("/Volumes");  
  File[] files = volumes.listFiles();  
  //println("Waiting...");
  for (int j=0; j < files.length; j++) {
    String mCurrentVolume = files[j].getName();             
    boolean mSuccess = false;
    for (int i=0; i < mOldVolumes.length; i++) {
      String mOldVolume = mOldVolumes[i].getName();
      if(mOldVolume.equals(mCurrentVolume)) {
        mSuccess = true;
      }
    }
    if (!mSuccess) {
      // mCurrentVolume is new
      println(mCurrentVolume + "is new.");
      USB = "/Volumes/" + mCurrentVolume;
      println("+++ dumping files from" + USB);
      File file = new File(USB);
      //We are going to wait here because there's a bit of lag or something
      //mysterious that stops the drawposter method from running properly.
      while(file.listFiles()==null) {
        try {
          Thread.sleep(100);
        } 
        catch(Exception e) {
          e.printStackTrace();
        }
      }
      makePoster(USB);             
      success = true;
    } 
  }
  return success;
}

// Function to get a list of all files in a directory and all subdirectories
ArrayList<File> listFilesRecursive(String dir) {
  ArrayList<File> fileList = new ArrayList<File>(); 
  recurseDir(fileList,dir);
  return fileList;
}

// Recursive function to traverse subdirectories
void recurseDir(ArrayList a, String dir) {
  println("+++ recursing dir" + dir);
  File file = new File(dir);
  println("+++ opening volume " + file.getName());
  if (file.isDirectory()) {
    // If you want to include directories in the list
    a.add(file);  
    File[] subfiles = file.listFiles((FileFilter) HiddenFileFilter.VISIBLE);
    println("+++ dumping subfiles" + subfiles);
    for (int i = 0; i < subfiles.length; i++) {
      // Call this function on all files in this directory
      recurseDir(a,subfiles[i].getAbsolutePath());
    }
  } 
  else {
    a.add(file);
  }
}

//Execute System CMD function
private boolean executeCMD(String[] cmd) { 
  boolean processed = false;
  try {
    Runtime rt = Runtime.getRuntime();
    Process pr = rt.exec(cmd);
    BufferedReader input = new BufferedReader(new InputStreamReader(pr.getErrorStream()));
    System.out.println("from exec");
    String line = null;
    System.out.println("<OUTPUT>");
    while ((line = input.readLine()) != null)
      System.out.println(line);
    System.out.println("</OUTPUT>");
    int exitVal = pr.waitFor();
    System.out.println("Exited with error code " + exitVal);
    pr.destroy();
    processed = true;
  } 
  catch (Throwable e) {
    e.printStackTrace();
  }
  return processed;
}

